@@ -1,21 +1,37 @@
-"""Macroeconomics
+'''
+.. module:: pymaclab
+   :platform: Linux
+   :synopsis: The var module contains the VAR class for estimating and doing further work with Vector Autoregressions commonly
+              used in applied macroeconometrics. It supports advanced methods such as bootstrapping confidence intervals including
+              Killian's boostrap-after-bootstrap small-sample bias correction. Also CPU-intensive methods such as the bootstrap can
+              be computed using Parallel Python to exploit multi-core CPUs. Pretty plotting methods are also included which depend
+              on matplotlib.
 
-__author__ = "Eric Michael Scheffel"
-__version__ = '0.8'
-__revision__ = "$Revision: 1001 $"
-__date__ = '$Date: 2007-08-05$'
+.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>
 
-"""
 
-__author__ = "Eric Michael Scheffel"
-__version__ = '0.88.1'
-__revision__ = "$Revision: 1001 $"
-__date__ = '$Date: 2012-08-21$'
+'''
 
 import os as OPS
 from dsge import macrolab
+from stats import var
+from stats import favar
 import linalg
 import sys
+import pp
+from copy import deepcopy
+
+# Expose the version number into library root
+from version import version as __version__
+
+# Start the job_server
+ppservers = ()
+jobserver = pp.Server(ppservers=ppservers)
+ncpus = jobserver.get_ncpus()
+del jobserver
+del ppservers
+print "PyMacLab started with parallel jobserver using "+str(ncpus)+" cores..."
+ 
 
 # Some helper functions
 def db_graph(dbase,tseries):
@@ -29,7 +45,8 @@ def db_graph(dbase,tseries):
 	fsp.legend()
 	P.show()
 
-def newMOD(txtfile=None,dbase=None,initlev=2,mesg=False,ncpus=1,mk_hessian=True,use_focs=False,ssidic=None):
+def newMOD(txtfile=None,dbase=None,initlev=2,mesg=False,ncpus=ncpus,\
+           mk_hessian=True,use_focs=False,ssidic=None,sstate=None):
 	'''
 	Model's second intialisation method called by newMOD() function call. The model's
 	__init__() method only creates the instance and adds information, but does no
@@ -40,8 +57,9 @@ def newMOD(txtfile=None,dbase=None,initlev=2,mesg=False,ncpus=1,mk_hessian=True,
 	init_lev = 1: parsing and steady state calculations
 	init_lev = 2: parsing, steady state calculations and dynamic solution computation
 	'''
-	modobj = macrolab.DSGEmodel(txtfile,dbase=dbase,initlev=initlev,mesg=mesg,ncpus=ncpus,
-	                            mk_hessian=mk_hessian,use_focs=use_focs,ssidic=ssidic)
+	modobj = macrolab.DSGEmodel(txtfile,dbase=dbase,initlev=initlev,mesg=mesg,ncpus=ncpus,\
+	                            mk_hessian=mk_hessian,use_focs=use_focs,\
+	                            ssidic=ssidic,sstate=sstate)
 	modobj.init1()
 	modobj.init1a()
 	modobj.init1b()
@@ -51,12 +69,16 @@ def newMOD(txtfile=None,dbase=None,initlev=2,mesg=False,ncpus=1,mk_hessian=True,
 	if initlev == 0:
 		modobj.init_out()
 		return modobj
-	# SS solved automatically
-	modobj.init3()
+	# SS solved automatically, but only if SS has not been handed over entirely externally
+	if sstate == None:
+		modobj.init3()
+	else:
+		if mesg: print "SS: Using entirely EXTERNALLY supplied steady state values dictionary..."
+		modobj.sstate = deepcopy(modobj._sstate)
 	if initlev == 1:
 		modobj.init_out()
 		return modobj
-	# Dynamic system prepared and solved
+	# Dynamic system prepared and prepared for solving
 	modobj.init4()
 	modobj.init5()
 	if initlev == 2:
@@ -66,8 +88,11 @@ def newMOD(txtfile=None,dbase=None,initlev=2,mesg=False,ncpus=1,mk_hessian=True,
 def newDB():
 	return macrolab.TSDataBase()
 
-def newVAR(laglen=1,data='none',set_useconst='const'):
-	return macrolab.VAR(laglen,data,set_useconst)
+def newVAR(data=None,vnames=None,pnames=None,svnames=None,irfs=True,boot=True,plot=True,conf=None,mesg=False):
+	return var.VAR(data=data,vnames=vnames,pnames=pnames,svnames=svnames,irfs=irfs,boot=boot,plot=plot,conf=conf,mesg=mesg)
+
+def newFAVAR(dates=None,rdates=None,data=None,freq='M',vnames=None,pnames=None,svnames=None,irfs=True,rescale=False,boot=True,plot=True,sfacs='auto',init=None,conf=None,mesg=False):
+	return favar.FAVAR(dates=dates,rdates=rdates,data=data,freq=freq,vnames=vnames,pnames=pnames,svnames=svnames,irfs=irfs,rescale=rescale,boot=boot,plot=plot,sfacs=sfacs,init=init,conf=conf,mesg=mesg)
 
 def modinfo(model):
 	offset = 40
@@ -266,8 +291,20 @@ def texedit(model):
 
 def explain(model):
 	model.pdf()
+	
+############ Make nost tests available###############
+def test():
+    try:
+	import nose
+	nose.run(defaultTest='pymaclab.tests')
+    except:
+	print "You need to install nose to run the tests."
+	print 'Run "sudo pip install nose" to install it.'
+    
+#####################################################
 
 
-# Import this late because cascading imports need newMOD() function'
+
+# Import this late because cascading imports need newMOD() function
 import modfiles
 from .modfiles.makemod import make_modfile
